const {
  AutojoinRoomsMixin,
  MatrixClient,
  RustSdkCryptoStorageProvider,
  SimpleFsStorageProvider,
} = require('matrix-bot-sdk');

const {
  MATRIX_HOMESERVER_URL,
  MATRIX_ACCESS_TOKEN,
  BOT_STORAGE_PATH,
  BOT_CRYPTO_STORAGE_PATH,
} = process.env;

const storageProvider = new SimpleFsStorageProvider(BOT_STORAGE_PATH);
const cryptoProvider = new RustSdkCryptoStorageProvider(BOT_CRYPTO_STORAGE_PATH);
console.log('MATRIX_HOMESERVER_URL:', MATRIX_HOMESERVER_URL);
const client = new MatrixClient(
  MATRIX_HOMESERVER_URL,
  MATRIX_ACCESS_TOKEN,
  storageProvider,
  cryptoProvider,
);
let joinedRooms;

const { startOutboundQueue } = require('./outbound-queue');

const start = async () => {
  // Automatically join a room to which we are invited
  AutojoinRoomsMixin.setupOnClient(client);

  joinedRooms = await client.getJoinedRooms();
  console.log('joined rooms:', joinedRooms);

  client.start().then(() => {
    console.log('Bot started!');
    // Start the outbound queue
    startOutboundQueue(client);
  });
};

const registerMessageHandler = (eventHandler) => {
  client.on('room.message', (roomId, event) => eventHandler(client, roomId, event));
};

const registerEventHandler = (eventHandler) => {
  client.on('room.event', (roomId, event) => {
    if (event.state_key !== undefined) return; // state event
    eventHandler(client, roomId, event);
  });
};

const getMatrixClient = () => client;

module.exports = {
  start,
  getMatrixClient,
  registerMessageHandler,
  registerEventHandler,
};
