const { getContractAddressByNetworkName } = require('../util/contract-config');
const { registerDecider } = require('./validation-pools');
const { registerEventHandler } = require('../matrix-bot');
const { matrixPools } = require('../util/db');

const {
  ETH_NETWORK,
} = process.env;

const rollupAddress = getContractAddressByNetworkName(ETH_NETWORK, 'Rollup');

const start = async () => {
  console.log('registering validation pool decider for rollup');
  registerDecider((pool, post) => {
    // If this is not sent by the work1 contract, it's not of interest here.
    if (pool.sender !== rollupAddress) return false;

    // A rollup post should contain
    // - a list of off-chain validation pools
    // - authorship corresponding to the result of those off-chain pools
    if (!post.embeddedData?.matrixPools) return false;

    // TODO: Compute expected result by fetching off-chain pool matrix events
    // TODO: Or precompute? Since we want to be computing that anyway.
    return false;
  });

  // We can use LevelDB to store information about validation pools

  registerEventHandler(async (client, roomId, event) => {
    switch (event.type) {
      case 'io.dgov.pool.start': {
        // Use the event id as the validation pool id
        const poolId = event.event_id;
        // TODO: This event should include validation pool params
        console.log('Matrix pool started', { poolId });
        await matrixPools.put(poolId, {});
        break;
      }
      case 'io.dgov.pool.stake': {
        // TODO: Keep track of stakes
        break;
      }
      case 'io.dgov.pool.result': {
        // This should be sent by the current batch worker
        // TODO: Compare batch worker's result with ours to verify and provide early warning
        break;
      }
      case 'io.dgov.rollup.submit': {
        // TODO: Compare batch worker's result with ours to verify
        break;
      }
      default:
    }
  });
};

module.exports = {
  start,
};
